"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dmmfModelsdeserializer = void 0;
const handlers = type => {
    return {
        default: value => {
            if (type === 'Boolean') {
                return `@default(${value})`;
            }
            if (!value) {
                return '';
            }
            if (typeof (value) === 'object') {
                return `@default(${value.name}(${value.args}))`;
            }
            if (typeof (value) === 'number') {
                return `@default(${value})`;
            }
            throw new Error(`Unsupporter field attribute ${value}`);
        },
        isId: value => value ? '@id' : '',
        isUnique: value => value ? '@unique' : '',
        dbNames: value => { },
        relationToFields: value => { },
        relationOnDelete: value => { },
        hasDefaultValue: value => { },
        relationName: value => { },
        isReadOnly: value => { },
        isGenerated: value => { },
        isUpdatedAt: value => value ? '@updatedAt' : '',
        columnName: value => value ? `@map("${value}")` : ''
    };
};
// Handler for Attributes
// https://www.prisma.io/docs/reference/tools-and-interfaces/prisma-schema/data-model#attributes
function handleAttributes(attributes, kind, type) {
    const { relationFromFields, relationToFields, relationName } = attributes;
    if (kind === 'scalar') {
        return `${Object.keys(attributes).map(each => handlers(type)[each](attributes[each])).join(' ')}`;
    }
    if (kind === 'object' && relationFromFields) {
        return relationFromFields.length > 0 ?
            `@relation(name: "${relationName}", fields: [${relationFromFields}], references: [${relationToFields}])` :
            `@relation(name: "${relationName}")`;
    }
    return '';
}
function handleFields(fields) {
    return fields
        .map(fields => {
        const { name, kind, type, isRequired, isList, ...attributes } = fields;
        if (kind === 'scalar') {
            return `  ${name} ${type}${isRequired ? '' : '?'} ${handleAttributes(attributes, kind, type)}`;
        }
        if (kind === 'object') {
            return `  ${name} ${type}${isList ? '[]' : (isRequired ? '' : '?')} ${handleAttributes(attributes, kind, type)}`;
        }
        throw new Error(`Unsupported field kind "${kind}"`);
    }).join('\n');
}
function handleUniqueFieds(uniqueFields) {
    return uniqueFields.length > 0 ? uniqueFields.map(eachUniqueField => `@@unique([${eachUniqueField.join(', ')}])`).join('\n') : '';
}
function handleDbName(dbName) {
    return dbName ? `@@map("${dbName}")` : '';
}
function deserializeModel(model) {
    const { name, uniqueFields, dbName } = model;
    const fields = model.fields;
    const output = `
model ${name} {
${handleFields(fields)}
${handleUniqueFieds(uniqueFields)}
${handleDbName(dbName)}
}`;
    return output;
}
/**
 * Deserialize DMMF.Model[] into prisma schema file
 */
async function dmmfModelsdeserializer(models) {
    return models.map(model => deserializeModel(model)).join('\n');
}
exports.dmmfModelsdeserializer = dmmfModelsdeserializer;
//# sourceMappingURL=deserializer.js.map